/*
 * BikesList.java
 *
 * Created on 12  2003 ., 13:27
 */

package com.fn.BikersLog;


import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import javax.swing.AbstractListModel;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;


/**
 *
 * @author  asso
 */
public class BikesList extends AbstractListModel {

    private List bikes;
    private Map allBikes;
    private ImageStorage imageStorage;
    
    public BikesList(ImageStorage is) {
        bikes = new ArrayList();
        allBikes = new TreeMap();
        imageStorage = is;
    }
    
    public Bike get(int index) {
        return (Bike)bikes.get(index);
    }
    
    public Bike getById(int id) {
        return (Bike)allBikes.get(new Integer(id));
    }
    
    public Object getElementAt(int index) {
        return get(index);
    }
    
    public int getSize() {
        return bikes.size();
    }
    
    protected void load(Element root) {
        if (root == null) return;
        NodeList elements = root.getElementsByTagName("bike");
        int size = elements.getLength();
        for (int i = 0; i < size; i++) {
            Bike b = new Bike(imageStorage, (Element)elements.item(i));
            if (! b.isDeleted())
                bikes.add(b);
            allBikes.put(new Integer(b.getId()), b);
        }
    }
    
    protected void save(Document document, Element root) {
        Element bikes = DOMUtil.addElement(document, root, "bikes");
        
        for (Iterator i = this.bikes.iterator(); i.hasNext(); )
            ((Bike)i.next()).save(document, bikes);
        
        for (Iterator i = allBikes.values().iterator(); i.hasNext(); ) {
            Bike b = (Bike)i.next();
            if (b.isDeleted())
                b.save(document, bikes);
        }
    }
    
    public void add(Bike bike) {
        bike.setId(getNextId());
        allBikes.put(new Integer(bike.getId()), bike);
        if (! bike.isDeleted()) {
            bikes.add(bike);
            fireIntervalAdded(this, bikes.size() - 1, bikes.size() - 1);
        }
    }
    
    public void change(int index) {
        fireContentsChanged(this, index, index);
    }
    
    public void delete(int index) {
        Bike b = get(index);
        b.setDeleted(true);
        bikes.remove(index);
        fireIntervalRemoved(this, index, index);
    }
    
    public int getNextId() {
        int maxId = 0;
        for (Iterator i = allBikes.values().iterator(); i.hasNext(); ) {
            Bike b = (Bike)i.next();
            int id = b.getId();
            if (id > maxId)
                maxId = id;
        }
        return maxId + 1;
    }
    
    public int getIndexOf(Bike bike) {
        return bikes.indexOf(bike);
    }
}
