/*
 * Course.java
 *
 * Created on 27  2003 ., 15:18
 */

package com.fn.BikersLog;

import org.w3c.dom.Text;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 *
 * @author  asso
 */
public class Course {

    private int id;
    private String name;
    private double length;
    private int difficulty;
    private int refCnt;
    private boolean deleted;
    private String notes;
    private int imageId;
    private ImageStorage imageStorage;
    private HeightProfile heightsProfile;
    
    /** Creates a new instance of Course */
    public Course(ImageStorage is) {
        imageStorage = is;
        name = "";
        length = 0.0;
        difficulty = 1;
        id = 0;
        refCnt = 0;
        deleted = false;
        notes = "";
        imageId = 0;
        heightsProfile = new HeightProfile();
    }
    
    public Course(ImageStorage is, Element course) throws Exception {
        imageStorage = is;
        name = DOMUtil.getSubtagData(course, "name", "");
        length = DOMUtil.getSubtagData(course, "length", 0.0);
        difficulty = DOMUtil.getSubtagData(course, "difficulty", 1);
        id = DOMUtil.getSubtagData(course, "id", 1);
        deleted = DOMUtil.getSubtagData(course, "deleted", false);
        notes = DOMUtil.getSubtagData(course, "notes", "");
        imageId = DOMUtil.getSubtagData(course, "image", 0);
        heightsProfile = new HeightProfile(DOMUtil.getTagByName(course, "profile"));
    }
    
    public void setName(String v) {
        name = v;
    }
    
    public String getName() {
        return name;
    }
    
    public double getLength() {
        if (! heightsProfile.isEmpty())
            return heightsProfile.getLength();
        else
            return length;
    }

    public void setLength(double length) {
        this.length = length;
    }
    
    public int getDifficulty() {
        return difficulty;
    }
    
    public void setDifficulty(int difficulty) {
        this.difficulty = difficulty;
    }    
    
    public String toString() {
        return name;
    }

    public void save(Document document, Element root) {
        if (deleted && (0 >= refCnt)) {
            if (0 != imageId)
                imageStorage.removeImage(imageId);
            return;
        }
        Element course = DOMUtil.addElement(document, root, "course");
        DOMUtil.addDataElement(document, course, "id", id);
        DOMUtil.addDataElement(document, course, "name", name);
        DOMUtil.addDataElement(document, course, "length", getLength());
        DOMUtil.addDataElement(document, course, "difficulty", difficulty);
        DOMUtil.addDataElement(document, course, "deleted", deleted);
        DOMUtil.addDataElement(document, course, "notes", notes);
        if (0 != imageId)
            DOMUtil.addDataElement(document, course, "image", imageId);
        if (! heightsProfile.isEmpty())
            heightsProfile.save(document, course, "profile");
    }
    
    public boolean isDeleted() {
        return deleted;
    }
    
    public void setDeleted(boolean v) {
        deleted = v;
    }
    
    public void addRef() {
        refCnt++;
    }
    
    public void delRef() {
        refCnt--;
    }
    
    public int getId() {
        return id;
    }
    
    public void setId(int v) {
        id = v;
    }
    
    public String getNotes() {
        return notes;
    }
    
    public void setNotes(String notes) {
        this.notes = notes;
    }
    
    public int getImage() {
        return imageId;
    }
    
    public void setImage(int image) {
        if (0 != imageId)
            imageStorage.removeImage(imageId);
        this.imageId = image;
    }
    
    public HeightProfile getHeightsProfile() {
        return heightsProfile;
    }
    
    public void setHeightsProfile(HeightProfile heightsProfile) {
        this.heightsProfile = heightsProfile;
    }
    
    public ImageStorage getImageStorage() {
        return imageStorage;
    }
}
