/*
 * CycleTrainerEvent.java
 *
 * Created on 2  2004 ., 11:13
 */

package com.fn.BikersLog;


import java.text.DateFormat;
import java.util.Date;
import javax.swing.ImageIcon;
import org.w3c.dom.Document;
import org.w3c.dom.Element;


/**
 *
 * @author  asso
 */
public class CycleTrainerEvent extends Event {
    
    private int heartRate;
    private String notes;
    private Date finish = new Date();
    private int effort;
    private static String typeName = I18n.getMsg("cycloTrainer");
    private static ImageIcon icon = null;
    private double wattage;
    private double weight;
    private double calories;
    
    /** Creates a new instance of CycleTrainerEvent */
    public CycleTrainerEvent() {
    }
    
    public CycleTrainerEvent(Element ride) {
        setStart(DOMUtil.getSubtagData(ride, "start", new Date()));
        finish = DOMUtil.getSubtagData(ride, "finish", new Date());
        heartRate = DOMUtil.getSubtagData(ride, "heartRate", 0);
        effort = DOMUtil.getSubtagData(ride, "effort", 0);
        notes = DOMUtil.getSubtagData(ride, "notes", "");
        wattage = DOMUtil.getSubtagData(ride, "wattage", 0.0);
        weight = DOMUtil.getSubtagData(ride, "weight", 0.0);
        calories = DOMUtil.getSubtagData(ride, "calories", 0.0);
    }
    
    public String toString() {
        DateFormat df = DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT);
        StringBuffer sb = new StringBuffer();
        sb.append(df.format(getStart())).append(' ').append(typeName);
        int elapsed = (int)(finish.getTime() - getStart().getTime()) / 1000;
        sb.append(' ').append(Utils.intervalToTime(elapsed));
        return sb.toString();
    }
    
    public void save(Document document, Element root) {
        Element ride = DOMUtil.addElement(document, root, "event");
        DOMUtil.addDataElement(document, ride, "type", "cyclotrainer");
        DOMUtil.addDataElement(document, ride, "start", getStart());
        DOMUtil.addDataElement(document, ride, "finish", finish);
        DOMUtil.addDataElement(document, ride, "heartRate", heartRate);
        DOMUtil.addDataElement(document, ride, "effort", effort);
        DOMUtil.addDataElement(document, ride, "notes", notes);
        DOMUtil.addDataElement(document, ride, "wattage", wattage);
        DOMUtil.addDataElement(document, ride, "weight", weight);
        DOMUtil.addDataElement(document, ride, "calories", calories);
    }
    
    public int getHeartRate() {
        return heartRate;
    }
    
    public void setHeartRate(int heartRate) {
        this.heartRate = heartRate;
    }
    
    public String getNotes() {
        return notes;
    }
    
    public void setNotes(String notes) {
        this.notes = notes;
    }
    
    public Date getFinish() {
        return finish;
    }
    
    public void setFinish(Date finish) {
        this.finish = finish;
    }
    
    public int getEffort() {
        return effort;
    }
    
    public void setEffort(int effort) {
        this.effort = effort;
    }    
    
    public ImageIcon getIcon() {
        if (null == icon)
            icon = new ImageIcon(getClass().getResource("/com/fn/BikersLog/img/trainer.png"));
        return icon;
    }
    
    public Object getColumnData(int column) {
        switch (column) {
            case Event.COLUMN_EFFORT: return new Integer(effort);
            case Event.COLUMN_ELAPSED: return Utils.intervalToTime(
                    (int)(finish.getTime() - getStart().getTime()) / 1000);
            case Event.COLUMN_HEART_RATE: return new Integer(heartRate);
            case Event.COLUMN_NOTES: return notes;
            case Event.COLUMN_TYPE: return typeName;
            case Event.COLUMN_WEIGHT:
                return Utils.doubleToStr(Units.fromStdWeight(weight));
            case Event.COLUMN_CALORIES:
                return Utils.doubleToStr(calories);
            case Event.COLUMN_WATTAGE:
                return Utils.doubleToStr(wattage);
            default:
                return super.getColumnData(column);
        }
    }
    
    public double getWattage() {
        return wattage;
    }
    
    public void setWattage(double wattage) {
        this.wattage = wattage;
    }
    
    public double getWeight() {
        return weight;
    }
    
    public void setWeight(double weight) {
        this.weight = weight;
    }
    
    public double getCalories() {
        return calories;
    }
    
    public void setCalories(double calories) {
        this.calories = calories;
    }
}
