/*
 * EventsList.java
 *
 * Created on 22  2003 ., 18:58
 */

package com.fn.BikersLog;


import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import javax.swing.AbstractListModel;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;


/**
 *
 * @author  asso
 */
public class EventsList extends AbstractListModel {

    private List events = new LinkedList();
    
    /** Creates a new instance of EventsList */
    public EventsList() {
    }
    
    protected void load(Element root, Log log) throws Exception {
        if (root == null) return;
        NodeList elements = root.getElementsByTagName("event");
        int size = elements.getLength();
        for (int i = 0; i < size; i++) {
            String type = DOMUtil.getSubtagData((Element)elements.item(i), "type", "");
            Event e = EventsFactory.loadEvent(type, (Element)elements.item(i), log);
            events.add(e);
        }
    }
    
    public Object getElementAt(int index) {
        return events.get(index);
    }
    
    public Event get(int index) {
        return (Event)events.get(index);
    }
    
    public int getSize() {
        return events.size();
    }
    
    public int add(Event event) {
        int index = -1;
        int pos = 0;
        long time = event.getStart().getTime();
        for (Iterator i = events.iterator(); i.hasNext() && (index == -1); ) {
            Event e = (Event)i.next();
            if (e.getStart().getTime() <= time)
                index = pos;
            pos++;
        }
        if (index == -1) {
            events.add(event);
            index = events.size() - 1;
        } else
            events.add(index, event);
        fireIntervalAdded(this, index, index);
        return index;
    }
    
    public int change(Event event) {
        int curIndex = events.indexOf(event);
        delete(curIndex);
        return add(event);
/*        if (curIndex < 0) return -1;
        int index = -1;
        int pos = 0;
        long time = event.getStart().getTime();
        for (Iterator i = events.iterator(); i.hasNext() && (index == -1); ) {
            Event e = (Event)i.next();
            if (e != event) {
                if (e.getStart().getTime() <= time)
                    index = pos;
            }
            pos++;
        }
        if (index == curIndex) {
            fireContentsChanged(this, index, index);
        } else {
            events.remove(curIndex);
            fireIntervalRemoved(this, curIndex, curIndex);
            if (index > curIndex)
                index--;
            events.add(index, event);
            fireIntervalAdded(this, index, index);
        }
        return index;*/
    }
    
    public void delete(int index) {
        Event event = get(index);
        events.remove(index);
        event.onRemove();
        fireIntervalRemoved(this, index, index);
    }
        
    public void save(Document document, Element root) {
        Element ev = DOMUtil.addElement(document, root, "events");
        
        for (Iterator i = events.iterator(); i.hasNext(); ) {
            Event e = (Event)i.next();
            e.save(document, ev);
        }
    }
    
    public void updateAll() {
        fireContentsChanged(this, 0, getSize());
    }
    
    public int getIndexOf(Event event) {
        return events.indexOf(event);
    }
}
