/*
 * FontChooserDialog.java
 *
 * Created on 26  2002 ., 17:03
 */

package com.fn.BikersLog;

import java.awt.Font;
import java.awt.GraphicsEnvironment;
import javax.swing.DefaultListModel;


/**
 *
 * @author  asso
 */
public class FontChooserDialog extends javax.swing.JDialog {

    private java.awt.Font selectedFont;
    private java.awt.Font currentFont;
    
    /** Creates new form FontChooserDialog */
    public FontChooserDialog(java.awt.Frame parent, boolean modal, Font oldFont) {
        super(parent, modal);
        init(oldFont);
        Utils.centerWindow(this, parent);
    }

    public FontChooserDialog(java.awt.Dialog parent, boolean modal, Font oldFont) {
        super(parent, modal);
        init(oldFont);
        Utils.centerWindow(this, parent);
    }
    
    private void init(Font oldFont) {
        initComponents();
        selectedFont = currentFont = null;
        
        int selectedIdx = fillFamilyList(oldFont);
        if (selectedIdx != -1)
            familyList.setSelectedIndex(selectedIdx);
        else
            familyList.setSelectedIndex(0);
        familyList.ensureIndexIsVisible(familyList.getSelectedIndex());

        fillStyleList();
        styleList.setSelectedIndex(fontStyleToIdx(oldFont));        
        styleList.ensureIndexIsVisible(styleList.getSelectedIndex());
        
        fillSizeList();
        sizeList.setSelectedIndex(fontSizeToIdx(oldFont));
        sizeList.ensureIndexIsVisible(sizeList.getSelectedIndex());
        
        setCurrentFont();
    }
    
    private int fillFamilyList(Font font) {
        GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
        String families[] = ge.getAvailableFontFamilyNames();
        DefaultListModel listModel = (DefaultListModel)familyList.getModel();
        String oldFamily = font.getFamily().toLowerCase();
        int selectedIdx = -1;
        for (int i = 0; i < families.length; i++) {
            listModel.addElement(families[i]);
            if (oldFamily.equals(families[i].toLowerCase()))
                selectedIdx = i;
        }
        return selectedIdx;
    }
    
    private void fillStyleList() {
        DefaultListModel listModel = (DefaultListModel)styleList.getModel();
        listModel.addElement(I18n.getMsg("fontChooser.style.normal"));
        listModel.addElement(I18n.getMsg("fontChooser.style.italic"));
        listModel.addElement(I18n.getMsg("fontChooser.style.bold"));
        listModel.addElement(I18n.getMsg("fontChooser.style.italicbold"));
    }
    
    private int fontStyleToIdx(Font font) {
        switch (font.getStyle()) {
            case Font.PLAIN: return 0;
            case Font.ITALIC: return 1;
            case Font.BOLD: return 2;
            default: return 3;
        }
    }
    
    private void fillSizeList() {
        DefaultListModel listModel = (DefaultListModel)sizeList.getModel();
        for (int i = 8; i <= 40; i+= 2) {
            listModel.addElement(Integer.toString(i));
        }
    }
    
    private int fontSizeToIdx(Font font) {
        int size = font.getSize();
        if (size <= 8)
            return 0;
        if (size >= 40)
            return (40 - 8) / 2;
        return (int)(size - 8) / 2;
    }
    
    private int indexToStyle(int idx) {
        switch (idx) {
            case 0: return Font.PLAIN;
            case 1: return Font.ITALIC;
            case 2: return Font.BOLD;
            default: return Font.ITALIC | Font.BOLD;
        }
    }
    
    private void setCurrentFont() {
        String family = (String)familyList.getModel().getElementAt(familyList.getSelectedIndex());
        int size = sizeList.getSelectedIndex() * 2 + 8;
        int style = indexToStyle(styleList.getSelectedIndex());
        java.util.Map map = new java.util.HashMap();        
        map.put(java.awt.font.TextAttribute.FAMILY, family);
        map.put(java.awt.font.TextAttribute.SIZE, new Float(size));
        if ((style & Font.ITALIC) != 0)
            map.put(java.awt.font.TextAttribute.POSTURE, java.awt.font.TextAttribute.POSTURE_OBLIQUE);
        else
            map.put(java.awt.font.TextAttribute.POSTURE, java.awt.font.TextAttribute.POSTURE_REGULAR);
        if ((style & Font.BOLD) != 0)
            map.put(java.awt.font.TextAttribute.WEIGHT, java.awt.font.TextAttribute.WEIGHT_BOLD);
        else
            map.put(java.awt.font.TextAttribute.WEIGHT, java.awt.font.TextAttribute.WEIGHT_REGULAR);
        currentFont = Font.getFont(map);
        sampleEdit.setFont(currentFont);
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        jPanel2 = new javax.swing.JPanel();
        jPanel8 = new javax.swing.JPanel();
        okBtn = new javax.swing.JButton();
        jPanel9 = new javax.swing.JPanel();
        cancelBtn = new javax.swing.JButton();
        jPanel1 = new javax.swing.JPanel();
        jPanel3 = new javax.swing.JPanel();
        jPanel5 = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        familyList = new javax.swing.JList();
        jLabel1 = new javax.swing.JLabel();
        jPanel6 = new javax.swing.JPanel();
        jScrollPane2 = new javax.swing.JScrollPane();
        styleList = new javax.swing.JList();
        jLabel2 = new javax.swing.JLabel();
        jPanel7 = new javax.swing.JPanel();
        jScrollPane3 = new javax.swing.JScrollPane();
        sizeList = new javax.swing.JList();
        jLabel3 = new javax.swing.JLabel();
        jPanel4 = new javax.swing.JPanel();
        sampleEdit = new javax.swing.JTextField();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle(I18n.getMsg("fontChooser.title"));
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });

        jPanel2.setLayout(new javax.swing.BoxLayout(jPanel2, javax.swing.BoxLayout.X_AXIS));

        jPanel2.setBorder(new javax.swing.border.EmptyBorder(new java.awt.Insets(4, 8, 8, 8)));
        jPanel2.add(jPanel8);

        okBtn.setText(I18n.getMsg("generic.ok"));
        okBtn.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                okBtnActionPerformed(evt);
            }
        });

        jPanel2.add(okBtn);

        jPanel9.setMaximumSize(new java.awt.Dimension(4, 4));
        jPanel9.setMinimumSize(new java.awt.Dimension(4, 4));
        jPanel9.setPreferredSize(new java.awt.Dimension(4, 4));
        jPanel2.add(jPanel9);

        cancelBtn.setText(I18n.getMsg("generic.cancel"));
        cancelBtn.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelBtnActionPerformed(evt);
            }
        });

        jPanel2.add(cancelBtn);

        getContentPane().add(jPanel2, java.awt.BorderLayout.SOUTH);

        jPanel1.setLayout(new java.awt.BorderLayout());

        jPanel1.setBorder(new javax.swing.border.EmptyBorder(new java.awt.Insets(8, 8, 4, 8)));
        jPanel3.setLayout(new javax.swing.BoxLayout(jPanel3, javax.swing.BoxLayout.X_AXIS));

        jPanel5.setLayout(new java.awt.BorderLayout());

        jScrollPane1.setVerticalScrollBarPolicy(javax.swing.JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        jScrollPane1.setPreferredSize(new java.awt.Dimension(180, 131));
        familyList.setModel(new javax.swing.DefaultListModel());
        familyList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        familyList.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                familyListValueChanged(evt);
            }
        });

        jScrollPane1.setViewportView(familyList);

        jPanel5.add(jScrollPane1, java.awt.BorderLayout.CENTER);

        jLabel1.setText(I18n.getMsg("fontChooser.family"));
        jPanel5.add(jLabel1, java.awt.BorderLayout.NORTH);

        jPanel3.add(jPanel5);

        jPanel6.setLayout(new java.awt.BorderLayout());

        jPanel6.setBorder(new javax.swing.border.EmptyBorder(new java.awt.Insets(0, 8, 0, 8)));
        jScrollPane2.setVerticalScrollBarPolicy(javax.swing.JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        jScrollPane2.setPreferredSize(new java.awt.Dimension(120, 131));
        styleList.setModel(new javax.swing.DefaultListModel());
        styleList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        styleList.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                familyListValueChanged(evt);
            }
        });

        jScrollPane2.setViewportView(styleList);

        jPanel6.add(jScrollPane2, java.awt.BorderLayout.CENTER);

        jLabel2.setText(I18n.getMsg("fontChooser.style"));
        jPanel6.add(jLabel2, java.awt.BorderLayout.NORTH);

        jPanel3.add(jPanel6);

        jPanel7.setLayout(new java.awt.BorderLayout());

        jScrollPane3.setVerticalScrollBarPolicy(javax.swing.JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        jScrollPane3.setPreferredSize(new java.awt.Dimension(40, 131));
        sizeList.setModel(new javax.swing.DefaultListModel());
        sizeList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        sizeList.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                familyListValueChanged(evt);
            }
        });

        jScrollPane3.setViewportView(sizeList);

        jPanel7.add(jScrollPane3, java.awt.BorderLayout.CENTER);

        jLabel3.setText(I18n.getMsg("fontChooser.size"));
        jPanel7.add(jLabel3, java.awt.BorderLayout.NORTH);

        jPanel3.add(jPanel7);

        jPanel1.add(jPanel3, java.awt.BorderLayout.CENTER);

        jPanel4.setLayout(new java.awt.BorderLayout());

        jPanel4.setBorder(new javax.swing.border.EmptyBorder(new java.awt.Insets(8, 0, 0, 0)));
        sampleEdit.setText(I18n.getMsg("fontChooser.sample"));
        sampleEdit.setPreferredSize(new java.awt.Dimension(69, 41));
        jPanel4.add(sampleEdit, java.awt.BorderLayout.CENTER);

        jPanel1.add(jPanel4, java.awt.BorderLayout.SOUTH);

        getContentPane().add(jPanel1, java.awt.BorderLayout.CENTER);

        pack();
    }//GEN-END:initComponents

    private void familyListValueChanged(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_familyListValueChanged
        setCurrentFont();
    }//GEN-LAST:event_familyListValueChanged

    private void cancelBtnActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cancelBtnActionPerformed
        dispose();
    }//GEN-LAST:event_cancelBtnActionPerformed

    private void okBtnActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_okBtnActionPerformed
        selectedFont = currentFont;
        dispose();
    }//GEN-LAST:event_okBtnActionPerformed
    
    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
        setVisible(false);
        dispose();
    }//GEN-LAST:event_closeDialog

    /** Return selected font or null if cancel pressed */
    public static java.awt.Font chooseFont(java.awt.Frame parent, Font oldFont) {
        FontChooserDialog fcd = new FontChooserDialog(parent, true, oldFont);
        fcd.setVisible(true);
        return fcd.selectedFont;
    }
        
    /** Return selected font or null if cancel pressed */
    public static java.awt.Font chooseFont(java.awt.Dialog parent, Font oldFont) {
        FontChooserDialog fcd = new FontChooserDialog(parent, true, oldFont);
        fcd.setVisible(true);
        return fcd.selectedFont;
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton cancelBtn;
    private javax.swing.JList familyList;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JPanel jPanel5;
    private javax.swing.JPanel jPanel6;
    private javax.swing.JPanel jPanel7;
    private javax.swing.JPanel jPanel8;
    private javax.swing.JPanel jPanel9;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JButton okBtn;
    private javax.swing.JTextField sampleEdit;
    private javax.swing.JList sizeList;
    private javax.swing.JList styleList;
    // End of variables declaration//GEN-END:variables
    
}
