/*
 * GroupedDataSet.java
 *
 * Created on 18  2004 ., 11:12
 */

package com.fn.BikersLog;


import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import com.fn.repway.DataSet;
import com.fn.repway.RepException;


/**
 *
 * @author  asso
 */
public abstract class GroupedDataSet implements DataSet {
    
    protected Object current;
    protected List groupedEvents;
    protected Iterator iterator;
    protected String name;
    
    /** Creates a new instance of GroupedDataSet */
    public GroupedDataSet(EventsList el, Date from, Date to, String name, 
            DateGrouper grouper, boolean addEmptyGroups) 
    {
        this.name = name;
        groupedEvents = new LinkedList();
        
        Date lastDate = null;
        List lastDateEvents = new LinkedList();
        Date lastGroupDate = grouper.getPrevGroupStart(from);
        long fromTime = from.getTime();
        long toTime = to.getTime();
        for (int i = el.getSize() - 1; i >= 0; i--) {
            Event event = el.get(i);
            if (isSuitableEvent(event)) {
                long time = event.getStart().getTime();
                if ((time >= fromTime) && (time <= toTime)) {
                    if ((lastDate != null) && (! grouper.isSameGroup(event.getStart(), lastDate))
                                && (0 < lastDateEvents.size())) 
                    {
                        addEvents(lastDateEvents, grouper.getGroupLabel(lastDate));
                        lastDateEvents.clear();
                    }
                    if ((! grouper.isSameGroup(lastGroupDate, event.getStart())) && addEmptyGroups) {
                        if (lastGroupDate.getTime() < event.getStart().getTime())
                            addEmptyEvents(grouper, lastGroupDate, event.getStart());
                    }
                    lastDate = event.getStart();
                    lastGroupDate = lastDate;
                    lastDateEvents.add(event);
                }
            }
        }
        if ((lastDate != null) && (0 < lastDateEvents.size()))
            addEvents(lastDateEvents, grouper.getGroupLabel(lastDate));
        if (addEmptyGroups) {
            Date f = lastGroupDate;
            Date t = grouper.getNextGroupStart(to);
            if ((f.getTime() < t.getTime()) && (! grouper.isSameGroup(f, t)))
                addEmptyEvents(grouper, f, t);
        }
        
        reset();
    }
    
    public String getName() {
        return name;
    }
    
    protected abstract boolean isSuitableEvent(Event event);
    
    protected abstract void addEmptyEvents(DateGrouper grouper, Date from, Date to);
    
    protected abstract void addEvents(List rides, String label);
    
    public boolean gotoNext() throws RepException {
        if (! iterator.hasNext()) {
            current = null;
            return false;
        }
        current = iterator.next();
        return true;
    }
    
    public boolean hasNext() throws RepException {
        return iterator.hasNext();
    }
    
    public boolean hasCurrent() throws RepException {
        return current != null;
    }    
    
    public void reset() {
        current = null;
        iterator = groupedEvents.iterator();
        try { gotoNext(); } catch (Exception e) { }
    }
}
