/*
 * DailyRidesDataSet.java
 *
 * Created on 6  2003 ., 22:39
 */

package com.fn.BikersLog;


import java.text.DateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import com.fn.repway.DataSet;
import com.fn.repway.RepException;
import com.fn.repway.Value;


/**
 *
 * @author  asso
 */
public class GroupedRidesDataSet extends GroupedDataSet {
    
    /** Creates a new instance of DailyRidesDataSet */
    public GroupedRidesDataSet(EventsList el, Date from, Date to, String name, 
            DateGrouper grouper, boolean addEmptyGroups) 
    {
        super(el, from, to, name, grouper, addEmptyGroups);
    }

    protected void addEmptyEvents(DateGrouper grouper, Date from, Date to) {
        Date d = grouper.getNextGroupStart(from);
        while (! grouper.isSameGroup(d, to)) {
            groupedEvents.add(new GroupedRide(grouper.getGroupLabel(d), 0.0, 0.0, 0));
            d = grouper.getNextGroupStart(d);
        }
    }
    
    protected void addEvents(List rides, String label) {
        int cnt = rides.size();
        if (cnt == 1) {
            RideEvent ride = (RideEvent)rides.get(0);
            groupedEvents.add(new GroupedRide(label, ride.getAvgSpeed(), 
                    ride.getDistance(), ride.getElapsed()));
        } else {
            double speed = 0.0, distance = 0.0;
            int time = 0;
            Date date = ((RideEvent)rides.get(0)).getStart();
            for (Iterator i = rides.iterator(); i.hasNext(); ) {
                RideEvent ride = (RideEvent)i.next();
                time += ride.getElapsed();
                distance += ride.getDistance();
                speed += ride.getAvgSpeed();
            }
            groupedEvents.add(new GroupedRide(label, speed / cnt, distance, time));
        }
    }
    
    private class GroupedRide {
        private String label;
        private double avgSpeed;
        private double distance;
        private int elapsed;
        
        public GroupedRide(String l, double v, double s, int t) {
            label = l;
            avgSpeed = Units.fromStdSpeed(v);
            distance = Units.fromStdBDist(s);
            elapsed = t;
        }
    }
    
    public Value getValue(String name) throws RepException {
        GroupedRide ride = (GroupedRide)current;
        if (name.equals("date"))
            return new Value(ride.label);
        else if (name.equals("distance"))
            return new Value(ride.distance);
        else if (name.equals("avgSpeed"))
            return new Value(ride.avgSpeed);
        else if (name.equals("elapsed"))
            return new Value(Value.TIMEINT, ride.elapsed);
        else
            throw new RepException("Unknown field '" + name + "'");
    }
    
    protected boolean isSuitableEvent(Event event) {
        return event instanceof RideEvent;
    }
    
}
