/*
 * DailyRidesDataSet.java
 *
 * Created on 6  2003 ., 22:39
 */

package com.fn.BikersLog;


import java.text.DateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import com.fn.repway.DataSet;
import com.fn.repway.RepException;
import com.fn.repway.Value;


/**
 *
 * @author  asso
 */
public class GroupedSkiingDataSet extends GroupedDataSet {
    
    /** Creates a new instance of DailyRidesDataSet */
    public GroupedSkiingDataSet(EventsList el, Date from, Date to, String name, 
            DateGrouper grouper, boolean addEmptyGroups) 
    {
        super(el, from, to, name, grouper, addEmptyGroups);
    }

    protected void addEmptyEvents(DateGrouper grouper, Date from, Date to) {
        Date d = grouper.getNextGroupStart(from);
        while (! grouper.isSameGroup(d, to)) {
            groupedEvents.add(new GroupedEvent(grouper.getGroupLabel(d), 0.0, 0.0, 0));
            d = grouper.getNextGroupStart(d);
        }
    }
    
    protected void addEvents(List events, String label) {
        int cnt = events.size();
        if (cnt == 1) {
            SkiingEvent event = (SkiingEvent)events.get(0);
            groupedEvents.add(new GroupedEvent(label, event.getAvgSpeed(), 
                    event.getDistance(), event.getElapsed()));
        } else {
            double speed = 0.0, distance = 0.0;
            int time = 0;
            Date date = ((Event)events.get(0)).getStart();
            for (Iterator i = events.iterator(); i.hasNext(); ) {
                SkiingEvent event = (SkiingEvent)i.next();
                time += event.getElapsed();
                distance += event.getDistance();
                speed += event.getAvgSpeed();
            }
            groupedEvents.add(new GroupedEvent(label, speed / cnt, distance, time));
        }
    }
    
    private class GroupedEvent {
        private String label;
        private double avgSpeed;
        private double distance;
        private int elapsed;
        
        public GroupedEvent(String l, double v, double s, int t) {
            label = l;
            avgSpeed = Units.fromStdSpeed(v);
            distance = Units.fromStdBDist(s);
            elapsed = t;
        }
    }
    
    public Value getValue(String name) throws RepException {
        GroupedEvent event = (GroupedEvent)current;
        if (name.equals("date"))
            return new Value(event.label);
        else if (name.equals("distance"))
            return new Value(event.distance);
        else if (name.equals("avgSpeed"))
            return new Value(event.avgSpeed);
        else if (name.equals("elapsed"))
            return new Value(Value.TIMEINT, event.elapsed);
        else
            throw new RepException("Unknown field '" + name + "'");
    }
    
    protected boolean isSuitableEvent(Event event) {
        return event instanceof SkiingEvent;
    }
    
}
