/*
 * HeightsView.java
 *
 * Created on 26  2004 ., 12:03
 */

package com.fn.BikersLog;


import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Graphics2D;
import java.awt.Point;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;


/**
 *
 * @author  asso
 */
public class HeightsView extends javax.swing.JComponent implements HeightProfileListener {
    
    private HeightProfile profile;
    private boolean edit = false;
    private double zoomX;
    private double zoomY;
    private double offset;
    private Cursor normalCursor = new Cursor(Cursor.DEFAULT_CURSOR);
    private Cursor crosshairCursor = new Cursor(Cursor.CROSSHAIR_CURSOR);
    private Cursor handCursor = new Cursor(Cursor.HAND_CURSOR);
    private List projected = new LinkedList();
    private ProjectedVertex dragging = null;
    private int lastDragX, lastDragY;
    private DragConstraint constraint;
    private JPopupMenu vertexPopup;
    private JPopupMenu linePopup;
    private ProjectedVertex selectedVertex;
    private int selectedX, selectedY;
    
    /** Creates a new instance of HeightsView */
    public HeightsView() {
        profile = null;
        init();
    }
    
    public HeightsView(HeightProfile heightProfile) {
        profile = heightProfile;
        if (null != profile)
            profile.addListener(this);
        init();
    }
    
    private void init() {
        updateZoom();
        updateProjected();
        
        addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentResized(java.awt.event.ComponentEvent e) {
                updateZoom();
                updateProjected();
            }
        });
        
        addMouseMotionListener(new java.awt.event.MouseMotionListener() {
            public void mouseMoved(MouseEvent e) {
                onMouseMoved(e);
            }
            
            public void mouseDragged(MouseEvent e) {
                onMouseDragged(e);
            }
        });
        
        addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(MouseEvent e) {
                onMousePressed(e);
            }
            
            public void mouseReleased(MouseEvent e) {
                onMouseReleased(e);
            }
            
            public void mouseClicked(MouseEvent e) {
                onMouseClicked(e);
            }
        });

        vertexPopup = new JPopupMenu();
        JMenuItem item = new JMenuItem(I18n.getMsg("generic.delete"));
        item.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent e) {
                onDeleteVertex();
            }
        });
        vertexPopup.add(item);
        item = new JMenuItem(I18n.getMsg("generic.properties"));
        item.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent e) {
                onEditVertex();
            }
        });
        vertexPopup.add(item);

        linePopup = new JPopupMenu();
        item = new JMenuItem(I18n.getMsg("heightsView.addVertex"));
        item.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent e) {
                onAddVertex();
            }
        });
        linePopup.add(item);
    }
    
    private void paintEmpty(Graphics2D g2) {
        super.paintComponent(g2);
        String msg = I18n.getMsg("heightsProfile.empty");
        int width = g2.getFontMetrics().stringWidth(msg);
        int height = g2.getFontMetrics().getHeight();
        int x = (getWidth() - width) / 2;
        int y = (getHeight() - height) / 2;
        g2.drawString(msg, x, y + g2.getFontMetrics().getAscent());
    }
    
    private void drawVertex(Graphics2D g2, int x, int y) {
        if (edit)
            g2.drawOval(x - 2, y - 2, 4, 4);
        g2.drawLine(x, y, x, getHeight());
    }
    
    private void drawSegment(Graphics2D g2, int x1, int y1, int x2, int y2, double slope) {
        int xPoly[] = { x1, x2, x2, x1 };
        int yPoly[] = { y1, y2, getHeight() - 1, getHeight() - 1 };
        float green = (float)(1.0f - slope / 40.0);
        if (0.0 > green)
            green = 0.0f;
        g2.setColor(new Color(0.0f, green, 0.0f));
        g2.fillPolygon(xPoly, yPoly, 4);
        g2.setColor(Color.BLACK);        
        g2.drawPolygon(xPoly, yPoly, 4);
        
        String s = Long.toString(Math.round(slope));
        int width = g2.getFontMetrics().stringWidth(s);
        if (width < x2 - x1)
            g2.drawString(s, (x2 - x1 - width) / 2 + x1, getHeight() - 4);
        
        if (edit) {
            g2.drawOval(x1 - 2, y1 - 2, 4, 4);
            g2.drawOval(x2 - 2, y2 - 2, 4, 4);
        }
    }
    
    protected void paintComponent(java.awt.Graphics g) {
        Graphics2D g2 = (Graphics2D)g;
        
        if ((null == profile) || (profile.isEmpty())) {
            paintEmpty(g2);
            return;
        }
        
        super.paintComponent(g2);

        Color oldBg = g2.getBackground();
        
        int lastX = 0, lastY = 0;
        
        boolean first = true;
        for (Iterator i = projected.iterator(); i.hasNext(); ) {
            ProjectedVertex pv = (ProjectedVertex)i.next();
            if (! first)
                drawSegment(g2, lastX, lastY, pv.x, pv.y, pv.slope);
            else
                first = false;
            lastX = pv.x;
            lastY = pv.y;
        }
        
        g2.setBackground(oldBg);
    }
    
    public boolean isEdit() {
        return edit;
    }
    
    public void setEdit(boolean edit) {
        this.edit = edit;
        repaint();
    }
    
    public HeightProfile getProfile() {
        return profile;
    }
    
    public void setProfile(HeightProfile profile) {
        if (null != this.profile)
            this.profile.removeListener(this);
        this.profile = profile;
        if (null != this.profile)
            this.profile.addListener(this);
        updateZoom();
        updateProjected();
    }
    
    public void onProfileChanged(HeightProfile profile) {
        updateZoom();
        updateProjected();
        repaint();
    }
    
    private void updateZoom() {
        offset = getHeight() - getHeight() / 4.0;
        if ((null == profile) || (profile.isEmpty())) {
            zoomX = 1.0;
            zoomY = 1.0;
        } else {
            double minHeight = profile.getMinHeight();
            double maxHeight = profile.getMaxHeight();
            double heightRange = maxHeight - minHeight;
            zoomY = heightRange != 0.0 ? getHeight() / heightRange / 2.0 : 1.0;
            if (zoomY > 1.0)
                zoomY = 1.0;
            offset = (int)(offset + minHeight * zoomY);
            double length = profile.getLength();
            zoomX = length > 0.0 ? (getWidth() - 5) / length : 1.0;
        }
    }
    
    private void mapScreenToVertex(Point2D.Double result, int x, int y) {
        result.x = (x - 2) / zoomX;
        result.y = (offset - y) / zoomY;
    }
    
    private void mapHeightToScreen(java.awt.Point result, HeightVertex vertex) {
        result.x = (int)(vertex.getPosition() * zoomX) + 2;
        result.y = (int)(offset - vertex.getHeight() * zoomY);
    }

    private ProjectedVertex getProjectedVertex(int mouseX, int mouseY) {
        for (Iterator i = projected.iterator(); i.hasNext(); ) {
            ProjectedVertex pv = (ProjectedVertex)i.next();
            if (pv.contains(mouseX, mouseY))
                return pv;
        }
        return null;
    }
    
    private void onMouseMoved(MouseEvent e) {
        if (! edit)
            return;
        
        ProjectedVertex pv = getProjectedVertex(e.getX(), e.getY());
        if (null != pv) {
            if (getCursor() != handCursor)
                setCursor(handCursor);
        } else {
            if (getCursor() != normalCursor)
                setCursor(normalCursor);
        }
    }

    private interface DragConstraint {
        public double apply(double value, double offset);
    }
    
    private class DontMoveConstraint implements DragConstraint {
        public DontMoveConstraint() {
        }
        
        public double apply(double value, double offset) {
            return value;
        }
    }

    private class RangeConstraint implements DragConstraint {
        private double min;
        private double max;
        
        public RangeConstraint(double min, double max) {
            this.min = min;
            this.max = max;
        }
        
        public double apply(double value, double offset) {
            double v = value + offset;
            if (v < min)
                return min;
            if (v > max)
                return max;
            return v;
        }
    }
    
    private void onMouseDragged(MouseEvent e) {
        if (null == dragging)
            return;
        
        int offsetX = e.getX() - lastDragX;
        int offsetY = e.getY() - lastDragY;
        
        dragging.vertex.setHeight(dragging.vertex.getHeight() - offsetY / zoomY);
        double v = offsetX / zoomX;
        dragging.vertex.setPosition(constraint.apply(dragging.vertex.getPosition(), v));
        dragging.update();
        
        lastDragX = e.getX();
        lastDragY = e.getY();
        
        repaint();
    }
    
    private class ProjectedVertex {
        private HeightVertex vertex;
        private int x;
        private int y;
        private int index;
        private double slope;
        
        public ProjectedVertex(HeightVertex v, int idx, double s) {
            Point p = new Point();
            mapHeightToScreen(p, v);
            x = p.x;
            y = p.y;
            vertex = v;
            index = idx;
            slope = s;
        }
        
        public boolean contains(int mouseX, int mouseY) {
            return (x >= mouseX - 2) && (x <= mouseX + 2) && 
                    (y >= mouseY - 2) && (y <= mouseY + 2);
        }
        
        public void update() {
            Point p = new Point();
            mapHeightToScreen(p, vertex);
            x = p.x;
            y = p.y;
        }
    }
    
    private void updateProjected() {
        projected.clear();
        
        if ((null == profile) || profile.isEmpty())
            return;
        
        int cnt = profile.getSize();
        Point p = new Point();
        HeightVertex pv = null;
        for (int i = 0; i < cnt; i++) {
            HeightVertex v = profile.getVertix(i);
            if (null != pv)
                projected.add(new ProjectedVertex(v, i, calcSlope(pv, v)));
            else
                projected.add(new ProjectedVertex(v, i, 0.0));
            pv = v;
        }
    }
    
    private double getNeighbourPos(ProjectedVertex vert, int offset) {
        return profile.getVertix(vert.index + offset).getPosition();
    }
    
    private void onMousePressed(MouseEvent e) {
        if ((! edit) || (null == profile) || (profile.isEmpty()))
            return;
        
        if (e.getButton() == MouseEvent.BUTTON1) {
            lastDragX = e.getX();
            lastDragY = e.getY();
            dragging = getProjectedVertex(lastDragX, lastDragY);
            if (null != dragging) {
                if (0 == dragging.index)
                    constraint = new DontMoveConstraint();
                else
                    if (profile.getSize() - 1 == dragging.index) {
                        double v = getNeighbourPos(dragging, -1);
                        constraint = new RangeConstraint(v, 
                            dragging.vertex.getPosition());
                    } else {
                        double v1 = getNeighbourPos(dragging, -1);
                        double v2 = getNeighbourPos(dragging, 1);
                        constraint = new RangeConstraint(v1, v2);
                    }
                setCursor(crosshairCursor);
            }
        }
    }
    
    private void onMouseReleased(MouseEvent e) {
        if (null != dragging) {
            dragging = null;
            /*updateZoom();
            updateProjected();
            repaint();*/
            profile.markAsUnordered();
            onMouseMoved(e);
        }
    }
    
    private void onVertexPopup(ProjectedVertex projectedVertex) {
        selectedVertex = projectedVertex;
        vertexPopup.show(this, projectedVertex.x, projectedVertex.y);
    }
    
    private void onLinePopup(int x, int y, ProjectedVertex v1, 
            ProjectedVertex v2) 
    {
        selectedX = x;
        selectedY = y;
        linePopup.show(this, x, y);
    }
    
    private boolean isPointOnLine(int x, int y, ProjectedVertex v1, 
            ProjectedVertex v2)
    {
        int x1 = v1.x;
        int x2 = v2.x;
        int y1 = v1.y;
        int y2 = v2.y;

        if (x1 == x2) {
            if (y1 < y2)
                return (y <= y2 + 2) && (y >= y1 - 2);
            else
                return (y <= y1 + 2) && (y >= y2 - 2);
        }
        
        double k;
        if (x2 == x1)
            k = 0.0;
        else
            k = ((double)(y2 - y1)) / ((double)(x2 - x1));
        int pY = (int)((x - x1) * k) + y1;
        
        return (y <= pY + 2) && (y >= pY - 2);
    }

    private void onMouseClicked(MouseEvent e) {
        if ((! edit) || (null == profile) || (profile.isEmpty()))
            return;
        
        if (e.getButton() == MouseEvent.BUTTON3) {
            ProjectedVertex pv = getProjectedVertex(e.getX(), e.getY());
            if (null != pv)
                onVertexPopup(pv);
            else {
                ProjectedVertex pv1 = null;
                ProjectedVertex pv2 = null;
                int x = e.getX();
                profile.order();
                for (Iterator i = projected.iterator(); i.hasNext(); ) {
                    pv1 = pv2;
                    pv2 = (ProjectedVertex)i.next();
                    if ((null != pv1) && (x >= pv1.x) && (x <= pv2.x)) {
                        if (isPointOnLine(x, e.getY(), pv1, pv2))
                            onLinePopup(x, e.getY(), pv1, pv2);
                        else
                            return;
                    }
                }
            }
        }
    }
    
    private void onAddVertex() {
        Point2D.Double p = new Point2D.Double();
        mapScreenToVertex(p, selectedX, selectedY);
        profile.addVertex(p.x, p.y);
    }
    
    private void onEditVertex() {
        java.awt.Dialog parent = Utils.findParentDialog(getParent());
        if (null != parent)
            if (HeightVertexDialog.edit(parent, selectedVertex.vertex))
                profile.markAsUnordered();
    }
    
    private void onDeleteVertex() {
        if ((0 == selectedVertex.index) || (profile.getSize() < 3))
            return;
        int res = JOptionPane.showConfirmDialog(Utils.findParentDialog(getParent()), 
                I18n.getMsg("heightsView.deleteConfirmation"),
                I18n.getMsg("mainForm.title"), JOptionPane.YES_NO_OPTION);
        if (res == JOptionPane.YES_OPTION)
            profile.deleteVertex(selectedVertex.index);
    }
    
    private double calcSlope(HeightVertex v1, HeightVertex v2) {
        double x1 = v1.getPosition();
        double y1 = v1.getHeight() / 1000.0;
        double x2 = v2.getPosition();
        double y2 = v2.getHeight() / 1000.0;
        
        double x = (x1 > x2) ? x1 - x2 : x2 - x1;
        double y = (y1 > y2) ? y1 - y2 : y2 - y1;
        
        if (0.0 == x)
            return 100.0;
        
        if (0.0 == y)
            return 0.0;

        return 100.0 * Math.atan2(y, x) / (Math.PI / 2.0);
    }
}
