/*
 * MonthlyEventsView.java
 *
 * Created on 23  2004 ., 13:26
 */

package com.fn.BikersLog;


import java.awt.BorderLayout;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import javax.swing.ComboBoxModel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.event.TableModelListener;
import javax.swing.event.TableModelEvent;


/**
 *
 * @author  asso
 */
public class MonthlyEventsPanel extends javax.swing.JPanel {
    
    private MonthlyEventsTableModel eventsModel;
    private EventsList eventsList;
    private String[] monthNames;
    private String[] yearsNames;
    private int baseYear;
    private boolean monitoring;
    private boolean dontUpdateModel = false;

    public MonthlyEventsPanel() {
        eventsList = new EventsList();
        init();
    }
    
    /** Creates new form MonthlyEventsView */
    public MonthlyEventsPanel(EventsList evList) {
        eventsList = evList;
        init();
    }

    private void init() {
        Calendar cal = Calendar.getInstance();
        baseYear = cal.get(Calendar.YEAR) - 100;
        createMonthNames();
        createYearsNames();
        eventsModel = new MonthlyEventsTableModel(eventsList);
        
        initComponents();
        
        daysTable.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent e) {
                if ((e.BUTTON1 == e.getButton()) && (e.getClickCount() == 2))
                    MainWindow.getInstance().editEvent();
            }
        });

        DefaultComboBoxModel model = (DefaultComboBoxModel)yearCombo.getModel();
        for (int i = baseYear; i < baseYear + 200; i++)
            yearCombo.addItem(new Integer(i).toString());
                
        daysTable.setDefaultRenderer(EventsInDay.class, new EventCellRenderer());
        javax.swing.table.JTableHeader tableHeader = daysTable.getTableHeader();
        tableHeader.setReorderingAllowed(false);
        tableHeader.setResizingAllowed(false);
        calendarPanel.add(tableHeader, BorderLayout.NORTH);
        selectToday();
        eventsModel.addTableModelListener(new TableModelListener() {
            public void tableChanged(TableModelEvent e) {
                if ((e.getType() == TableModelEvent.UPDATE) &&
                        (e.getColumn() == TableModelEvent.ALL_COLUMNS))
                    daysTableComponentResized(null);
            }
        });
        
        monitoring = false;
    }
    
    public void startMonitoring() {
        eventsList.addListDataListener(eventsModel);
        monitoring = true;
    }
    
    public void stopMonitoring() {
        eventsList.removeListDataListener(eventsModel);
        monitoring = false;
    }

    private void createMonthNames() {
        monthNames = new String[12];
        SimpleDateFormat sdf = new SimpleDateFormat("MMMMM");
        Calendar cal = Calendar.getInstance();
        cal.set(Calendar.DATE, 1);
        int j = 0;
        for (int i = Calendar.JANUARY; i <= Calendar.DECEMBER; i++) {
            cal.set(Calendar.MONTH, i);
            monthNames[j] = sdf.format(cal.getTime());
            j++;
        }
    }

    private void createYearsNames() {
        yearsNames = new String[200];
        for (int i = 0; i < 200; i++)
            yearsNames[i] = Integer.toString(baseYear + i);
    }
    
    public void setEventsList(EventsList list) {
        if (monitoring)
            eventsList.removeListDataListener(eventsModel);
        eventsList = list;
        eventsModel.setEventsList(eventsList);
        if (monitoring)
            eventsList.addListDataListener(eventsModel);
        selectToday();
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        jPanel1 = new javax.swing.JPanel();
        todayBtn = new javax.swing.JButton();
        jPanel3 = new javax.swing.JPanel();
        prevMonthBtn = new javax.swing.JButton();
        monthCombo = new javax.swing.JComboBox(monthNames);
        nextMonthBtn = new javax.swing.JButton();
        jPanel2 = new javax.swing.JPanel();
        prevYearBtn = new javax.swing.JButton();
        yearCombo = new javax.swing.JComboBox(yearsNames);
        nextYearBtn = new javax.swing.JButton();
        calendarPanel = new javax.swing.JPanel();
        daysTable = new javax.swing.JTable();

        setLayout(new java.awt.BorderLayout());

        todayBtn.setText(I18n.getMsg("monthView.today"));
        todayBtn.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                todayBtnActionPerformed(evt);
            }
        });

        jPanel1.add(todayBtn);

        jPanel3.setPreferredSize(new java.awt.Dimension(20, 10));
        jPanel1.add(jPanel3);

        prevMonthBtn.setText("<");
        prevMonthBtn.setMargin(new java.awt.Insets(2, 2, 2, 2));
        prevMonthBtn.setMinimumSize(new java.awt.Dimension(10, 10));
        prevMonthBtn.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                prevMonthBtnActionPerformed(evt);
            }
        });

        jPanel1.add(prevMonthBtn);

        monthCombo.setMaximumRowCount(12);
        monthCombo.setMinimumSize(new java.awt.Dimension(82, 24));
        monthCombo.setPreferredSize(new java.awt.Dimension(82, 24));
        monthCombo.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                monthComboActionPerformed(evt);
            }
        });

        jPanel1.add(monthCombo);

        nextMonthBtn.setText(">");
        nextMonthBtn.setMargin(new java.awt.Insets(2, 2, 2, 2));
        nextMonthBtn.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                nextMonthBtnActionPerformed(evt);
            }
        });

        jPanel1.add(nextMonthBtn);

        jPanel2.setPreferredSize(new java.awt.Dimension(20, 10));
        jPanel1.add(jPanel2);

        prevYearBtn.setText("<");
        prevYearBtn.setMargin(new java.awt.Insets(2, 2, 2, 2));
        prevYearBtn.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                prevYearBtnActionPerformed(evt);
            }
        });

        jPanel1.add(prevYearBtn);

        yearCombo.setMaximumRowCount(11);
        yearCombo.setMinimumSize(new java.awt.Dimension(82, 24));
        yearCombo.setPreferredSize(new java.awt.Dimension(82, 24));
        yearCombo.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                yearComboActionPerformed(evt);
            }
        });

        jPanel1.add(yearCombo);

        nextYearBtn.setText(">");
        nextYearBtn.setMargin(new java.awt.Insets(2, 2, 2, 2));
        nextYearBtn.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                nextYearBtnActionPerformed(evt);
            }
        });

        jPanel1.add(nextYearBtn);

        add(jPanel1, java.awt.BorderLayout.NORTH);

        calendarPanel.setLayout(new java.awt.BorderLayout());

        daysTable.setModel(eventsModel);
        daysTable.setRowSelectionAllowed(false);
        daysTable.addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentResized(java.awt.event.ComponentEvent evt) {
                daysTableComponentResized(evt);
            }
        });

        calendarPanel.add(daysTable, java.awt.BorderLayout.CENTER);

        add(calendarPanel, java.awt.BorderLayout.CENTER);

    }//GEN-END:initComponents

    public void selectToday() {
        Calendar cal = Calendar.getInstance();
        eventsModel.setMonth(cal.getTime());
        int row = eventsModel.getDayRow(cal.get(Calendar.DATE));
        int column = eventsModel.getDayColumn(cal.get(Calendar.DATE));
        daysTable.changeSelection(row, column, false, false);
        updateLabels(true);
    }
    
    private void todayBtnActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_todayBtnActionPerformed
        selectToday();
    }//GEN-LAST:event_todayBtnActionPerformed

    private void yearComboActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_yearComboActionPerformed
        if (dontUpdateModel)
            return;
        Calendar cal = Calendar.getInstance();
        cal.setTime(eventsModel.getMonth());
        cal.set(Calendar.YEAR, baseYear + yearCombo.getSelectedIndex());
        eventsModel.setMonth(cal.getTime());
        updateLabels(true);
    }//GEN-LAST:event_yearComboActionPerformed

    private void monthComboActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_monthComboActionPerformed
        if (dontUpdateModel)
            return;
        Calendar cal = Calendar.getInstance();
        cal.setTime(eventsModel.getMonth());
        cal.set(Calendar.MONTH, monthCombo.getSelectedIndex());
        eventsModel.setMonth(cal.getTime());
        updateLabels(true);
    }//GEN-LAST:event_monthComboActionPerformed

    private void addViewDate(int field, int value) {
        Calendar cal = Calendar.getInstance();
        int year = cal.get(Calendar.YEAR);
        int month = cal.get(Calendar.MONTH);
        int day = cal.get(Calendar.DATE);
        cal.setTime(eventsModel.getMonth());
        cal.add(field, value);
        int newYear = cal.get(Calendar.YEAR);
        if ((newYear < baseYear) || (newYear >= baseYear + 200))
            return;
        eventsModel.setMonth(cal.getTime());
        updateLabels(false);
        if ((year == newYear) && (month == cal.get(Calendar.MONTH))) {
            int row = eventsModel.getDayRow(day);
            int column = eventsModel.getDayColumn(day);
            daysTable.changeSelection(row, column, false, false);
        }
    }
    
    public void updateView() {
        eventsModel.fireTableDataChanged();
    }
    
    private void nextMonthBtnActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_nextMonthBtnActionPerformed
        addViewDate(Calendar.MONTH, 1);
    }//GEN-LAST:event_nextMonthBtnActionPerformed

    private void nextYearBtnActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_nextYearBtnActionPerformed
        addViewDate(Calendar.YEAR, 1);
    }//GEN-LAST:event_nextYearBtnActionPerformed

    private void prevMonthBtnActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_prevMonthBtnActionPerformed
        addViewDate(Calendar.MONTH, -1);
    }//GEN-LAST:event_prevMonthBtnActionPerformed

    private void updateLabels(boolean quite) {
        boolean oldDum = dontUpdateModel;
        dontUpdateModel = quite;
        Calendar cal = Calendar.getInstance();
        cal.setTime(eventsModel.getMonth());
        monthCombo.setSelectedIndex(cal.get(Calendar.MONTH));
        yearCombo.setSelectedIndex(cal.get(Calendar.YEAR) - baseYear);
        dontUpdateModel = oldDum;
    }
    
    private void prevYearBtnActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_prevYearBtnActionPerformed
        addViewDate(Calendar.YEAR, -1);
    }//GEN-LAST:event_prevYearBtnActionPerformed

    private void daysTableComponentResized(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_daysTableComponentResized
        int h = daysTable.getRowHeight();
        int newH = daysTable.getHeight() / daysTable.getRowCount();
        if (h != newH)
            daysTable.setRowHeight(newH);
    }//GEN-LAST:event_daysTableComponentResized
    
    public EventsInDay getSelected() {
        int row = daysTable.getSelectedRow();
        int column = daysTable.getSelectedColumn();
        return eventsModel.getEventsInDay(row, column);
    }
    
    public Date getSelectedDate() {
        int row = daysTable.getSelectedRow();
        int column = daysTable.getSelectedColumn();
        Date date = eventsModel.getDate(row, column);
        if (null == date)
            return new Date();
        Calendar nowCal = Calendar.getInstance();
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        cal.set(Calendar.HOUR, nowCal.get(Calendar.HOUR));
        cal.set(Calendar.MINUTE, nowCal.get(Calendar.MINUTE));
        cal.set(Calendar.SECOND, nowCal.get(Calendar.SECOND));
        return cal.getTime();
    }
    
    public void focusTable() {
        daysTable.requestFocus();
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel calendarPanel;
    private javax.swing.JTable daysTable;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JComboBox monthCombo;
    private javax.swing.JButton nextMonthBtn;
    private javax.swing.JButton nextYearBtn;
    private javax.swing.JButton prevMonthBtn;
    private javax.swing.JButton prevYearBtn;
    private javax.swing.JButton todayBtn;
    private javax.swing.JComboBox yearCombo;
    // End of variables declaration//GEN-END:variables
    
}
