/*
 * RunRoute.java
 *
 * Created on 14  2004 ., 10:24
 */

package com.fn.BikersLog;


import org.w3c.dom.Document;
import org.w3c.dom.Element;


/**
 *
 * @author  asso
 */
public class RunRoute {
    
    private int id;
    private String name;
    private double length;
    private int refCnt;
    private boolean deleted;
    private String notes;
    private int imageId;
    private ImageStorage imageStorage;
    private HeightProfile heightsProfile;

    /** Creates a new instance of RunRoute */
    public RunRoute(ImageStorage is) {
        imageStorage = is;
        name = "";
        length = 0.0;
        id = 0;
        refCnt = 0;
        deleted = false;
        notes = "";
        imageId = 0;
        heightsProfile = new HeightProfile();
    }
    
    public RunRoute(ImageStorage is, Element route) throws Exception {
        imageStorage = is;
        name = DOMUtil.getSubtagData(route, "name", "");
        length = DOMUtil.getSubtagData(route, "length", 0.0);
        id = DOMUtil.getSubtagData(route, "id", 1);
        deleted = DOMUtil.getSubtagData(route, "deleted", false);
        notes = DOMUtil.getSubtagData(route, "notes", "");
        imageId = DOMUtil.getSubtagData(route, "image", 0);
        heightsProfile = new HeightProfile(DOMUtil.getTagByName(route, "profile"));
    }
    
    public void setName(String v) {
        name = v;
    }
    
    public String getName() {
        return name;
    }
    
    public double getLength() {
        if (! heightsProfile.isEmpty())
            return heightsProfile.getLength();
        else
            return length;
    }

    public void setLength(double length) {
        this.length = length;
    }
    
    public String toString() {
        return name;
    }

    public void save(Document document, Element root) {
        if (deleted && (0 >= refCnt)) {
            if (0 != imageId)
                imageStorage.removeImage(imageId);
            return;
        }
        Element route = DOMUtil.addElement(document, root, "runRoute");
        DOMUtil.addDataElement(document, route, "id", id);
        DOMUtil.addDataElement(document, route, "name", name);
        DOMUtil.addDataElement(document, route, "length", getLength());
        DOMUtil.addDataElement(document, route, "deleted", deleted);
        DOMUtil.addDataElement(document, route, "notes", notes);
        if (0 != imageId)
            DOMUtil.addDataElement(document, route, "image", imageId);
        if (! heightsProfile.isEmpty())
            heightsProfile.save(document, route, "profile");
    }
    
    public boolean isDeleted() {
        return deleted;
    }
    
    public void setDeleted(boolean v) {
        deleted = v;
    }
    
    public void addRef() {
        refCnt++;
    }
    
    public void delRef() {
        refCnt--;
    }
    
    public int getId() {
        return id;
    }
    
    public void setId(int v) {
        id = v;
    }
    
    public String getNotes() {
        return notes;
    }
    
    public void setNotes(String notes) {
        this.notes = notes;
    }
    
    public int getImage() {
        return imageId;
    }
    
    public void setImage(int image) {
        if (0 != imageId)
            imageStorage.removeImage(imageId);
        this.imageId = image;
    }
    
    public HeightProfile getHeightsProfile() {
        return heightsProfile;
    }
    
    public void setHeightsProfile(HeightProfile heightsProfile) {
        this.heightsProfile = heightsProfile;
    }
    
    public ImageStorage getImageStorage() {
        return imageStorage;
    }
}
