/*
 * PreferencesDialog.java
 *
 * Created on 8  2004 ., 12:55
 */

package com.fn.BikersLog.prefs;


import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import com.fn.BikersLog.Config;
import com.fn.BikersLog.I18n;
import com.fn.BikersLog.Utils;


/**
 *
 * @author  asso
 */
public class PreferencesDialog extends javax.swing.JDialog {
    
    private static PreferencesTree pages;
    private boolean okPressed;
    private PreferencesPage currentPage;
    private Map usedPages;
    
    static {
        pages = new PreferencesTree();
        pages.addPage("root", "general", "preferencesForm.general.title", GeneralPane.class);
        pages.addPage("root", "views", "preferencesForm.views", EmptyPane.class);
        pages.addPage("views", "grid", "preferencesForm.tableView", TableViewPane.class);
        pages.addPage("views", "calendar", "preferencesForm.monthView", MonthViewPane.class);
    }
    
    /** Creates new form PreferencesDialog */
    public PreferencesDialog(java.awt.Frame parent) {
        super(parent, true);
        okPressed = false;
        usedPages = new HashMap();
        initComponents();
        for (int row = 0; row < pagesTree.getRowCount(); row++)
            pagesTree.expandRow(row);
        pagesTree.setSelectionRow(0);
        Utils.centerWindow(this, parent);
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        jPanel1 = new javax.swing.JPanel();
        okBtn = new javax.swing.JButton();
        cancelBtn = new javax.swing.JButton();
        jPanel2 = new javax.swing.JPanel();
        jSplitPane1 = new javax.swing.JSplitPane();
        jScrollPane1 = new javax.swing.JScrollPane();
        pagesTree = new javax.swing.JTree(pages);
        pagePanel = new javax.swing.JPanel();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle(I18n.getMsg("preferencesForm.title"));
        jPanel1.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.CENTER, 8, 8));

        okBtn.setText(I18n.getMsg("generic.ok"));
        okBtn.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                okBtnActionPerformed(evt);
            }
        });

        jPanel1.add(okBtn);

        cancelBtn.setText(I18n.getMsg("generic.cancel"));
        cancelBtn.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelBtnActionPerformed(evt);
            }
        });

        jPanel1.add(cancelBtn);

        getContentPane().add(jPanel1, java.awt.BorderLayout.SOUTH);

        jPanel2.setLayout(new java.awt.BorderLayout());

        jPanel2.setBorder(new javax.swing.border.EmptyBorder(new java.awt.Insets(8, 8, 0, 8)));
        jPanel2.setPreferredSize(new java.awt.Dimension(550, 400));
        jSplitPane1.setDividerLocation(120);
        pagesTree.setCellRenderer(new PrefsCellRenderer());
        pagesTree.setRootVisible(false);
        pagesTree.addTreeSelectionListener(new javax.swing.event.TreeSelectionListener() {
            public void valueChanged(javax.swing.event.TreeSelectionEvent evt) {
                pagesTreeValueChanged(evt);
            }
        });

        jScrollPane1.setViewportView(pagesTree);

        jSplitPane1.setLeftComponent(jScrollPane1);

        pagePanel.setLayout(new java.awt.BorderLayout());

        pagePanel.setMinimumSize(new java.awt.Dimension(100, 100));
        pagePanel.setPreferredSize(new java.awt.Dimension(100, 100));
        jSplitPane1.setRightComponent(pagePanel);

        jPanel2.add(jSplitPane1, java.awt.BorderLayout.CENTER);

        getContentPane().add(jPanel2, java.awt.BorderLayout.CENTER);

        pack();
    }//GEN-END:initComponents

    private void pagesTreeValueChanged(javax.swing.event.TreeSelectionEvent evt) {//GEN-FIRST:event_pagesTreeValueChanged
        if (null != currentPage)
            currentPage.remove(pagePanel);
        currentPage = null;
        javax.swing.tree.TreePath path = evt.getPath();
        if (null != path) {
            PrefsPageCreator creator = (PrefsPageCreator)path.getLastPathComponent();
            PreferencesPage page = (PreferencesPage)usedPages.get(creator.getIdent());
            if (null != page) {
                page.install(pagePanel);
                currentPage = page;
            } else {
                try {
                    page = creator.createPage();
                    page.install(pagePanel);
                    usedPages.put(creator.getIdent(), page);
                    currentPage = page;
                } catch (Exception e) {
                    javax.swing.JOptionPane.showMessageDialog(this, e.getMessage());
                }
            }
        }
        pagePanel.validate();
        pagePanel.repaint();
    }//GEN-LAST:event_pagesTreeValueChanged

    private void cancelBtnActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cancelBtnActionPerformed
        setVisible(false);
    }//GEN-LAST:event_cancelBtnActionPerformed

    private void okBtnActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_okBtnActionPerformed
        Config.disableNotifiers();
        for (Iterator i = usedPages.values().iterator(); i.hasNext(); )
            ((PreferencesPage)i.next()).apply();
        Config.enableNotifiers();
        setVisible(false);
    }//GEN-LAST:event_okBtnActionPerformed
    
    public static void execute(javax.swing.JFrame owner) {
        PreferencesDialog pw = new PreferencesDialog(owner);
        pw.setVisible(true);
        pw.dispose();
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton cancelBtn;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JSplitPane jSplitPane1;
    private javax.swing.JButton okBtn;
    private javax.swing.JPanel pagePanel;
    private javax.swing.JTree pagesTree;
    // End of variables declaration//GEN-END:variables
    
}
